"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.customElementsJSONGenerator = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const utils_1 = require("../../utils");
const validators_1 = require("../validators");
const messages_1 = require("../messages");
const formatter = (text) => text.replace(/fast-/g, 'foundation-').replace(/FAST /g, '');
const combineSchemas = (componentConfig, tagSchema, field) => (componentConfig[field] || [])
    .map((authoringSchema) => {
    const fastSchema = (0, utils_1.getByName)(tagSchema[field], authoringSchema.name);
    if (!fastSchema) {
        console.warn(`${field} doesn't exist in the FAST schema`, authoringSchema);
        return null;
    }
    const description = formatter((authoringSchema && authoringSchema.description) || fastSchema.description || '');
    return Object.assign(Object.assign(Object.assign({}, fastSchema), authoringSchema), { description });
})
    .filter((s) => !!s);
const getEnabledComponents = () => {
    const config = (0, utils_1.getUIAuthoringConfig)();
    return config.components.filter((c) => !c.disabled);
};
exports.customElementsJSONGenerator = {
    description: (0, messages_1.generatorDescription)('Generate a custom element schema file for the foundation-ui base components.'),
    prompts(inquirer, prevAns = {}) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const enabledComponents = getEnabledComponents();
            const { components } = yield inquirer.prompt([
                {
                    type: 'checkbox',
                    message: 'Which components should this include?',
                    name: 'components',
                    choices: (0, utils_1.mapToCheckboxChoices)(enabledComponents, 'name', 'name'),
                    default: (0, utils_1.mapToNames)(enabledComponents, 'name'),
                    validate: validators_1.atLeastOne,
                },
            ]);
            return {
                components,
            };
        });
    },
    actions: (answers) => {
        const schema = {
            version: 1.1,
            tags: [],
        };
        console.log();
        answers.components.forEach((name) => {
            const componentConfig = (0, utils_1.getByName)(getEnabledComponents(), name);
            const fastComponentName = componentConfig.fastComponentName || name.replace(/foundation-/g, '');
            console.log(`Processing ${name}:`);
            try {
                const componentSchema = (0, utils_1.getBaseComponentSchema)(fastComponentName);
                console.log(`* Loaded FAST component schema for: ${fastComponentName}`);
                const tagSchema = componentSchema.implementations[0].implementation.tags[0];
                const tag = {
                    name: componentConfig.name,
                    title: tagSchema.title,
                    description: formatter(tagSchema.description),
                    childComponent: componentConfig.childComponent,
                    properties: [],
                    events: [],
                    methods: [],
                    cssProperties: [],
                    cssParts: [],
                    attributes: combineSchemas(componentConfig, tagSchema, 'attributes'),
                    slots: combineSchemas(componentConfig, tagSchema, 'slots'),
                };
                console.log(`* Augmented the FAST component schema with authoring configuration`);
                schema.tags.push(tag);
            }
            catch (e) {
                console.error(`Failed to process ${name} (FAST component name: ${fastComponentName})`, e);
            }
        });
        return [
            () => {
                const indent = 2;
                const content = JSON.stringify(schema, null, indent);
                const filePath = path_1.default.resolve(__dirname, `../../../../../foundation/foundation-ui/custom-elements.json`);
                fs_1.default.writeFileSync(filePath, content);
                return `File written to ${filePath}`;
            },
        ];
    },
};
//# sourceMappingURL=customElements.js.map